#define PIN_PUSHBUTTON 	4
#define PIN_IR			12

#define DEBOUNCEDELAY 500

// TVの電源のON／OFF
short ARRAY_HIGHLOW[] = { 2492, 508, 1280, 500, 688, 504, 1280, 500, 688, 504, 1284, 524, 740, 460, 680, 496, 1284, 508, 708, 472,
                          716, 480, 684, 504, 656, 25500, 2492, 500, 1280, 508, 684, 500, 1280, 508, 688, 500, 1280, 508, 704, 484,
                          708, 476, 1308, 508, 676, 508, 660, 528, 712, 476, 688, 25456, 2496, 508, 1272, 508, 680, 504, 1284, 504,
                          680, 508, 1280, 524, 684, 508, 664, 528, 1276, 508, 680, 504, 692, 496, 744, 460, 672, };

void setup()
{ 
	pinMode(PIN_PUSHBUTTON, INPUT);
	pinMode(PIN_IR, OUTPUT);
}

void loop()
{
	static int s_iButtonState_prev = LOW;
	static unsigned long s_ulTime_prev = 0;

	int iButtonState = digitalRead(PIN_PUSHBUTTON);
	
	if( LOW == s_iButtonState_prev
	 && HIGH == iButtonState )
	{	// OFF→ON時
		unsigned long ulTime = millis();

		if( DEBOUNCEDELAY < (ulTime - s_ulTime_prev) )
		{	// デバウンス時間よりも経過しているならば、正当な「OFF→ON」とみなす
			sendSignal();
		}

		s_ulTime_prev = ulTime;
	}

	s_iButtonState_prev = iButtonState;
}

void sendSignal()
{
	int iTimeLength;
	unsigned long ulTimeStart;
	
	int iCountHighLow = sizeof( ARRAY_HIGHLOW ) / sizeof( ARRAY_HIGHLOW[0] );
	for( int iIndexHighLow = 0; iIndexHighLow < iCountHighLow; iIndexHighLow++ )
	{
		iTimeLength = ARRAY_HIGHLOW[iIndexHighLow];
		ulTimeStart = micros();
		do
		{
			digitalWrite( PIN_IR, 1 - (iIndexHighLow % 2) );	// iIndexHighLow : 0は、high。 1は、low。以下同様。
			delayMicroseconds(8);	// 周波数38kHzでOn/Offするように２つのdelayMicrosecondsの時間を調整する。
									// 厳密には、38kHz => 26.3[microseconds]/回だが、digitalWriteの時間があるので、数字通りにはいかない。
			digitalWrite( PIN_IR, 0 );
			delayMicroseconds(7);
		} while( iTimeLength > micros() - ulTimeStart );

	}
}
